#include "map.h"

game_map::game_map(RNG &rng, const feature enabled_features):
        enabled_features{enabled_features} {

}

game_map::game_map(const std::string &map_data,
                   RNG &rng, const feature enabled_features):
        enabled_features{enabled_features} {
        int map_size = MAP_HEIGHT * MAP_WIDTH;
        map.reserve(map_size);

        for (int i = 0; i < map_size; ++i) {
                if (map_data[i] >= '0' && map_data[i] <= '9')
                        map.push_back(map_data[i] - '0');
                else
                        map.push_back(map_data[i]);

                if ((map_data[i] >= '0' && map_data[i] <= '9') ||
                    map_data[i] == '+' || map_data[i] == '#')
                        empty_spots.push_back(remap_index(i));
        }

        down_stairs = rng.get_rand_in_vector(empty_spots);


        if (enabled_features | FEATURE_REVISIT) {
                up_stairs = rng.get_rand_in_vector(
                                    remove_from_list(empty_spots, {down_stairs}));
                map[remap_position(up_stairs)] = '<';
                map[remap_position(down_stairs)] = '>';
        } else {
                map[remap_position(down_stairs)] = '\\';
        }
}

const position_list game_map::get_available_positions() const {
        return empty_spots;
}

void game_map::print(display &out) const {
        for (std::size_t i = 0; i < map.size(); ++i)
                out.print_char(remap_index(i), map[i] <= 9 ? '.' : map[i],
                               map[i] == '\\' || map[i] == '>' ||
                               map[i] == '<' ? COLOR_PAIR(COLOR_BLUE) : A_NORMAL);
}

position game_map::get_up_stairs() const {
        return up_stairs;
}

position game_map::get_down_stairs() const {
        return down_stairs;
}

int game_map::get_up_stairs_room() const {
        return map[remap_position(up_stairs)];
}

int game_map::get_down_stairs_room() const {
        return map[remap_position(down_stairs)];
}

const std::vector<position_list>game_map::get_room_list() const {
        std::vector<position_list> result;
        result.reserve(10);

        for (int i = 0; i < 10; ++i)
                result.push_back({});

        for (std::size_t i = 0; i < map.size(); ++i)
                if (map[i] <= 9)
                        result[map[i]].push_back(remap_index(i));

        result.shrink_to_fit();

        for (std::size_t i = 0; i < result.size(); ++i)
                result[i].shrink_to_fit();

        return result;
}
