/*
 * CS 246 Final Project
 * File: map.h
 * Purpose: handles map functionality
 */

#ifndef __MAP_H__
#define __MAP_H__
#include <vector>
#include <string>
#include "constants.h"
#include "display.h"
#include "position.h"
#include "characters.h"
#include "rng.h"

class game_map final {
private:
        const feature enabled_features;
        std::vector<char> map;
        // IMPORTANT: keep empty_spots ordered
        position_list empty_spots;
        position up_stairs;
        position down_stairs;
        int room_cnt;
public:
        // randomly generate a map
        game_map(RNG &rng, const feature enabled_features);
        // read map from a string
        game_map(const std::string &map_data, RNG &rng,
                 const feature enabled_features);

        const position_list get_available_positions() const;

        const std::vector<position_list> get_room_list() const;
        // IMPORTANT: always print a map before anything else
        void print(display &out) const;

        position get_up_stairs() const;
        position get_down_stairs() const;

        int get_up_stairs_room() const;
        int get_down_stairs_room() const;
private:
        position remap_index(const int idx) const {
                return {idx % MAP_WIDTH, idx / MAP_HEIGHT};
        }

        int remap_position(const position &pos) const {
                return pos.y * MAP_WIDTH + pos.x;
        }
};

const std::string default_map =
        "|-----------------------------------------------------------------------------|\
|                                                                             |\
| |--------------------------|        |-----------------------|               |\
| |11111111111111111111111111|        |33333333333333333333333|               |\
| |11111111111111111111111111+########+33333333333333333333333|-------|       |\
| |11111111111111111111111111|   #    |3333333333333333333333333333333|--|    |\
| |11111111111111111111111111|   #    |3333333333333333333333333333333333|--| |\
| |----------+---------------|   #    |----+----------------|333333333333333| |\
|            #                 #############                |333333333333333| |\
|            #                 #     |-----+------|         |333333333333333| |\
|            #                 #     |444444444444|         |333333333333333| |\
|            ###################     |444444444444|   ######+333333333333333| |\
|            #                 #     |444444444444|   #     |333333333333333| |\
|            #                 #     |-----+------|   #     |--------+------| |\
|  |---------+-----------|     #           #          #              #        |\
|  |222222222222222222222|     #           #          #         |----+------| |\
|  |222222222222222222222|     ########################         |00000000000| |\
|  |222222222222222222222|     #           #                    |00000000000| |\
|  |222222222222222222222|     #    |------+--------------------|00000000000| |\
|  |222222222222222222222|     #    |000000000000000000000000000000000000000| |\
|  |222222222222222222222+##########+000000000000000000000000000000000000000| |\
|  |222222222222222222222|          |000000000000000000000000000000000000000| |\
|  |---------------------|          |---------------------------------------| |\
|                                                                             |\
|-----------------------------------------------------------------------------|";

#endif
